import { AxiosRequestConfig, AxiosResponse } from "axios";
import { Checker, IsType } from "@paulpopat/safe-type";
declare type Metadata = {
    base: string;
    headers?: {
        [key: string]: string;
    };
    on_request?: (v: AxiosRequestConfig) => Promise<AxiosRequestConfig>;
    on_response?: (value: AxiosResponse) => Promise<AxiosResponse>;
    on_error?: (error: any, route: string) => Error;
};
declare type BodyType<TBody, TReturns> = {
    url: string;
    parameters: {
        [key: string]: Checker<string | string[] | null | undefined>;
    };
    returns: Checker<TReturns>;
    method: "PUT" | "POST" | "PATCH";
    body: Checker<TBody>;
};
declare type UrlType<TReturns> = {
    url: string;
    parameters: {
        [key: string]: Checker<string | string[] | null | undefined>;
    };
    returns: Checker<TReturns>;
    method: "GET" | "DELETE";
};
declare type PlainUrlType<TReturns> = {
    url: string;
    returns: Checker<TReturns>;
    method: "GET" | "DELETE";
};
declare type PlainBodyType<TBody, TReturns> = {
    url: string;
    returns: Checker<TReturns>;
    method: "PUT" | "POST" | "PATCH";
    body: Checker<TBody>;
};
declare type Apis = {
    [key: string]: BodyType<any, any> | UrlType<any> | PlainUrlType<any> | PlainBodyType<any, any> | Apis;
};
declare type UrlFunction<TReturns, TSchema extends UrlType<TReturns>> = (parameters: {
    [key in keyof TSchema["parameters"]]: IsType<TSchema["parameters"][key]>;
}) => Promise<IsType<TSchema["returns"]>>;
declare type BodyFunction<TBody, TReturns, TSchema extends BodyType<TBody, TReturns>> = (parameters: {
    [key in keyof TSchema["parameters"]]: IsType<TSchema["parameters"][key]>;
}, body: IsType<TSchema["body"]>) => Promise<IsType<TSchema["returns"]>>;
declare type PlainUrlFunction<TReturns, TSchema extends PlainUrlType<TReturns>> = () => Promise<IsType<TSchema["returns"]>>;
declare type PlainBodyFunction<TBody, TReturns, TSchema extends PlainBodyType<TBody, TReturns>> = (body: IsType<TSchema["body"]>) => Promise<IsType<TSchema["returns"]>>;
declare type ApiInterface<TSchema extends Apis> = {
    [key in keyof TSchema]: TSchema[key] extends BodyType<any, any> ? BodyFunction<TSchema[key]["body"], TSchema[key]["returns"], TSchema[key]> : TSchema[key] extends UrlType<any> ? UrlFunction<TSchema[key]["returns"], TSchema[key]> : TSchema[key] extends PlainUrlType<any> ? PlainUrlFunction<TSchema[key]["returns"], TSchema[key]> : TSchema[key] extends PlainBodyType<any, any> ? PlainBodyFunction<TSchema[key]["body"], TSchema[key]["returns"], TSchema[key]> : TSchema[key] extends Apis ? ApiInterface<TSchema[key]> : unknown;
};
export default function <TSchema extends Apis>(schema: TSchema, metadata: Metadata): ApiInterface<TSchema> & {
    headers: {
        [key: string]: string;
    };
};
export {};
